searchNodes=[{"doc":"Flexible and composable authorization for resources defined by an Ecto.Schema . Janus provides an API for defining authorization policies that can be used both as filters in Ecto queries and to authorize actions on loaded resources with minimal to no duplication of authorization logic. Janus is split into two primary components: Janus.Policy - functions and behaviour for defining policy modules , which describe the allowed actors, actions, and resources in your application. This is where you look if you're writing a policy module. Janus.Authorization - functions and behaviour used by the rest of your application to authorize and load resources. This is where you look if you're using a policy module. Janus defines a Mix task to generate the basic policy module that will get you started: $ mix janus.gen.policy Policies Policy modules are created by invoking use Janus , which implements both the Janus.Policy and Janus.Authorization behaviours: defmodule Policy do use Janus @impl true def policy_for ( policy , _user ) do policy end end When you invoke use Janus , default implementations are injected for required callbacks, except for Janus.Policy.policy_for/2 . This callback is your foundation, as it returns the authorization policy for an individual user of your application. The policy above is not very useful -- it doesn't allow anyone to do anything -- but that can be changed by using the Janus.Policy API to define actions, resources, and conditions that make up your authorization rules. def policy_for ( policy , % User { role : :moderator } = mod ) do policy |&gt; allow ( :read , Post ) |&gt; allow ( [ :edit , :archive , :unarchive ] , Post , where : [ user : [ role : :member ] ] ) |&gt; allow ( [ :edit , :archive , :unarchive ] , Post , where : [ user_id : mod . id ] ) |&gt; deny ( :unarchive , Post , where : [ archived_by : [ role : :admin ] ] ) end See the Janus.Policy documentation for more on defining policies. Authorization With our policy module defined, it can now be used to load and authorize resources. iex&gt; Policy . authorize ( some_post , :archive , moderator ) { :ok , some_post } iex&gt; Policy . authorize ( post_archived_by_admin , :unarchive , moderator ) :error iex&gt; Policy . filter_authorized ( Post , :read , moderator ) % Ecto.Query { } iex&gt; Policy . filter_authorized ( Post , :read , moderator ) |&gt; Repo . all ( ) [ ... posts the moderator can read ] iex&gt; Policy . any_authorized? ( Post , :edit , moderator ) true # there are rules allowing moderators to edit posts iex&gt; Policy . any_authorized? ( Post , :delete , moderator ) false # there are no rules that allow moderators to delete posts These functions make up the Janus.Authorization behaviour, and their definitions were injected by default when we invoked use Janus . This is the &quot;public API&quot; that the rest of your application will use to authorize resources. See the Janus.Authorization documentation for more. Integration with Ecto The primary assumption that Janus makes is that your resources are backed by an Ecto.Schema . Using Ecto's schema reflection capabilities, Janus is able to use the same policy to authorize a single resource and to construct a composable Ecto query that is aware of field types and associations. # This query would result in the 5 latest posts that the current # user is authorized to see, preloaded with the user # who made # the post (but only if the current user is allowed to see that # user). Post |&gt; Policy . filter_authorized ( :read , current_user , preload_authorized : :user ) |&gt; order_by ( desc : :inserted_at ) |&gt; limit ( 5 ) This integration with Ecto queries is main reason Janus exists.","ref":"Janus.html","title":"Janus","type":"module"},{"doc":"Sets up a module to implement the Janus.Policy and Janus.Authorization behaviours. Invoking use Janus does the following: invokes use Janus.Policy which imports functions for defining policies and injects wrapper definitions for policy_for/1 and policy_for/2 that support hooks (see Janus.Policy for more) injects implementations for the Janus.Authorization behaviour Example defmodule MyApp.Policy do use Janus @impl true def policy_for ( policy , _actor ) do policy end end # imports `authorize`, `filter_authorized`, and `any_authorized?` use MyApp.Policy","ref":"Janus.html#__using__/1","title":"Janus.__using__/1","type":"macro"},{"doc":"","ref":"Janus.html#t:action/0","title":"Janus.action/0","type":"type"},{"doc":"","ref":"Janus.html#t:actor/0","title":"Janus.actor/0","type":"type"},{"doc":"","ref":"Janus.html#t:schema_module/0","title":"Janus.schema_module/0","type":"type"},{"doc":"Authorize and load resources using policies. Policy modules expose a minimal API that can be used to authorize and load resources throughout the rest of your application. authorize/4 - authorize an individual, already-loaded resource filter_authorized/4 - construct an Ecto query for a schema that will filter results to only those that are authorized any_authorized?/3 - checks whether the given actor/policy has any access to the given schema for the given action These functions will usually be called from your policy module directly, since wrappers that accept either a policy or an actor are injected when you invoke use Janus . Documentation examples will show usage from your policy module. See individual function documentation for details.","ref":"Janus.Authorization.html","title":"Janus.Authorization","type":"behaviour"},{"doc":"Checks whether any permissions are defined for the given schema, action, and actor. This function is most useful in conjunction with filter_authorized/4 , which builds an Ecto query that filters to only those resources the actor is authorized for. If you run the resulting query and receive [] , it is not possible to determine whether the result is empty because the actor wasn't authorized for any resources or because of other restrictions on the query. For example, you might use the following pattern to load all the resources a user is allowed to read that were inserted in the last day: query = from ( r in MyResource , where : r . inserted_at &gt; from_now ( - 1 , &quot;day&quot; ) ) if any_authorized? ( query , :read , user ) do { :ok , filter_authorized ( query , :read , user ) |&gt; Repo . all ( ) } else :error end This would result in {:ok, results} if the user is authorized to read any resources, even if the result set is empty, and would result in :error if the user isn't authorized to read the resources at all. Examples iex&gt; MyPolicy . any_authorized? ( MyResource , :read , actor ) true iex&gt; MyPolicy . any_authorized? ( MyResource , :delete , actor ) false","ref":"Janus.Authorization.html#any_authorized?/3","title":"Janus.Authorization.any_authorized?/3","type":"function"},{"doc":"","ref":"Janus.Authorization.html#c:any_authorized?/3","title":"Janus.Authorization.any_authorized?/3","type":"callback"},{"doc":"Authorizes a loaded resource. Expects to receive a struct, an action, and an actor or policy. Returns {:ok, resource} if authorized, otherwise :error . Examples iex&gt; MyPolicy . authorize ( % MyResource { } , :read , actor ) # accepts an actor { :ok , % MyResource { } } iex&gt; MyPolicy . authorize ( % MyResource { } , :read , policy ) # or a policy { :ok , % MyResource { } } iex&gt; MyPolicy . authorize ( % MyResource { } , :delete , actor ) :error","ref":"Janus.Authorization.html#authorize/4","title":"Janus.Authorization.authorize/4","type":"function"},{"doc":"","ref":"Janus.Authorization.html#c:authorize/4","title":"Janus.Authorization.authorize/4","type":"callback"},{"doc":"Create an %Ecto.Query{} that results in only authorized records. Like the Ecto.Query API, this function can accept a schema as the first argument or a query, in which case it will compose with that query. If a query is passed, the appropriate schema will be derived from that query's source. filter_authorized ( MyResource , :read , user ) query = from ( r in MyResource , where : r . inserted_at &gt; from_ago ( 1 , &quot;day&quot; ) ) filter_authorized ( query , :read , user ) If the query specifies the source as a string, we cannot derive the schema. For example, this will not work: # Raises an ArgumentError query = from ( r in &quot;my_resources&quot; , where : r . inserted_at &gt; from_ago ( 1 , &quot;day&quot; ) ) filter_authorized ( query , :read , user ) Options :preload_authorized - Similar to Ecto.Query.preload/3 , but only preloads those associated records that are authorized. Note that this requires Ecto v3.9.4 or later and a database that supports lateral joins. See &quot;Preloading authorized associations&quot; for more information. Preloading authorized associations The :preload_authorized option can be used to preload associated records, but only those that are authorized for the given actor. An additional query can be specified for each preloaded association that will be run as if scoped to its parent row. This can simplify certain queries dramatically. For instance, imagine a user search interface that lists users along with their most recent comment. Naughty comments can be hidden by moderators, but those hidden comments should still be visible if a moderator is searching. Here's how that might be accomplished: iex&gt; last_comment = from ( Comment , order_by : [ desc : :inserted_at ] , limit : 1 ) iex&gt; User ...&gt; |&gt; search ( search_params ) ...&gt; |&gt; MyPolicy . filter_authorized ( :read , current_user , ...&gt; preload_authorized : [ comments : last_comment ] ...&gt; ) ...&gt; |&gt; Repo . all ( ) [ % User { comments : [ % Comment { } ] } , % User { comments : [ % Comment { } ] } , ... ] Some things to note about this example: The last_comment query runs as if scoped to each user's comments. This means that the :limit applies to each user's comments, not the entire set of comments. The comment will be the last inserted comment that is authorized to be read by the current_user . Moderators may be able to see hidden comments, while normal users may not. It is also possible to nest authorized preloads. For instance, you could preload comments and their associated post. MyPolicy . filter_authorized ( User , :read , current_user , preload_authorized : [ comments : :post ] ) This would load all comments. You could incorporate the last_comment query above by specifying it as the first element of a tuple, followed by the list of inner preloads: MyPolicy . filter_authorized ( User , :read , current_user , preload_authorized : [ comments : { last_comment , [ :post ] } ] ) This would load only the latest comment as well as its associated post (assuming it too is authorized to be read by current_user ). Examples iex&gt; MyPolicy . filter_authorized ( MyResource , :read , actor ) % Ecto.Query { } iex&gt; MyPolicy . filter_authorized ( MyResource , :read , actor ) |&gt; Repo . all ( ) [ % MyResource { } , ... ] iex&gt; MyResource ...&gt; |&gt; MyPolicy . filter_authorized ( :read , actor ) ...&gt; |&gt; order_by ( inserted_at : :desc ) ...&gt; |&gt; limit ( 1 ) ...&gt; |&gt; Repo . one ( ) % MyResource { } iex&gt; MyResource ...&gt; |&gt; MyPolicy . filter_authorized ( :read , actor , ...&gt; preload_authorized : :other ...&gt; ) ...&gt; |&gt; Repo . all ( ) [ % MyResource { other : % OtherResource { } } , ... ]","ref":"Janus.Authorization.html#filter_authorized/4","title":"Janus.Authorization.filter_authorized/4","type":"function"},{"doc":"","ref":"Janus.Authorization.html#c:filter_authorized/4","title":"Janus.Authorization.filter_authorized/4","type":"callback"},{"doc":"","ref":"Janus.Authorization.html#t:filterable/0","title":"Janus.Authorization.filterable/0","type":"type"},{"doc":"Define composable authorization policies for actors in your system. A policy is a data structure created for an actor in your system that defines the schemas that actor can access, the actions they can take, and any restrictions to the set of resources that can be accessed. These policies are generally created implicitly for actors passed to functions defined by Janus.Authorization , but they can also be created (and cached) with policy_for/2 . Policy modules While you can create a policy module with use Janus.Policy , you will usually invoke use Janus to create a module that implements both this and the Janus.Authorization behaviour: defmodule MyApp.Policy do use Janus @impl true def policy_for ( policy , _user ) do policy end end The policy_for/2 callback is the only callback that is required in policy modules. Using allow and deny Permissions are primarily defined using allow/4 and deny/4 , which allows or denies an action on a resource if a set of conditions match. Both functions take the same arguments and options. When permissions are being checked, multiple allow rules combine using logical-or, with deny rules overriding allow . For example, the following policy would allow a moderator to edit their own comments and any comments flagged for review, but not those made by an admin. def policy_for ( policy , % User { role : :moderator } = user ) do policy |&gt; allow ( :edit , Comment , where : [ user : [ id : user . id ] ] ) |&gt; allow ( :edit , Comment , where : [ flagged_for_review : true ] ) |&gt; deny ( :edit , Comment , where : [ user : [ role : :admin ] ] ) end While set of keyword options passed to allow and deny are reminiscent of keyword-based Ecto queries, but since they are functions and not macros, there is no need to use the ^value syntax used in Ecto. For example, the following would result in an error: allow ( policy , :edit , Comment , where : [ user : [ id : ^ user . id ] ] ) :where and :where_not conditions These conditions match if the associated fields are equal to each other. For instance, the moderation example above could also be written as: def policy_for ( policy , % User { role : :moderator } = user ) do policy |&gt; allow ( :edit , Comment , where : [ user_id : user . id ] ) |&gt; allow ( :edit , Comment , where : [ flagged_for_review : true ] , where_not : [ user : [ role : :admin ] ] ) end Multiple conditions within the same allow / deny are combined with a logical-and, so this might be translated to English as &quot;allow moderators to edit comments they made or to edit comments flagged for review that were not made by an admin&quot;. :or_where conditions You can also use :or_where to combine with all previous conditions. For instance, the two examples above could also be written as: def policy_for ( policy , % User { role : :moderator } = user ) do policy |&gt; allow ( :edit , Comment , where : [ flagged_for_review : true ] , where_not : [ user : [ role : :admin ] ] , or_where : [ user_id : user . id ] ) end An :or_where condition applies to all clauses before it. Using some pseudocode for demonstration, the above would read: # (flagged_for_review AND NOT user.role == :admin) OR user_id == user.id These clauses could be reordered to have a different meaning: policy |&gt; allow ( :edit , Comment , where : [ flagged_for_review : true ] , or_where : [ user_id : user . id ] , where_not : [ user : [ role : :admin ] ] ) # (flagged_for_review OR user_id == user.id) AND NOT user.role == :admin Attribute checks with functions When equality is not a sufficient check for an attribute, a function can be supplied. For instance, a published_at field might be used to schedule posts. Users may only have permission to read posts where published_at is in the past, but we can only check for equality using the basic keyword syntax presented above. In these cases, you can defer this check using an arity-3 function: def policy_for ( policy , user ) do policy |&gt; allow ( :read , Post , where : [ published_at : &amp; in_the_past? / 3 ] ) end def in_the_past? ( :boolean , record , :published_at ) do if value = Map . get ( record , :published_at ) do DateTime . compare ( DateTime . utc_now ( ) , value ) == :gt end end def in_the_past? ( :dynamic , binding , :published_at ) do now = DateTime . utc_now ( ) Ecto.Query . dynamic ( ^ now &gt; as ( ^ binding ) . published_at ) end As seen in the example above, functions must define at least two clauses based on their first argument, :boolean or :dynamic , so that they can handle both operations on a single record and operations that should compose with an Ecto query. before_policy_for hooks You can register hooks to be run prior to policy_for/2 using before_policy_for/1 . These hooks can be used to change the default (usually empty) policy or actor, or to prevent policy_for/2 from being run altogether. See before_policy_for/1 for more details.","ref":"Janus.Policy.html","title":"Janus.Policy","type":"behaviour"},{"doc":"Allows an action on the schema if matched by conditions. See the section on &quot;Using allow and deny&quot; for a description of conditions. Examples policy |&gt; allow ( :read , FirstResource ) |&gt; allow ( :create , SecondResource , where : [ creator : [ id : user . id ] ] )","ref":"Janus.Policy.html#allow/4","title":"Janus.Policy.allow/4","type":"function"},{"doc":"Specifies that a condition should match if another action is allowed. If used as the value for an association, the condition will match if the action is allowed for the association. Examples Allow users to edit any posts they can delete. policy |&gt; allow ( :edit , Post , where : allows ( :delete ) ) |&gt; allow ( :delete , Post , where : [ user_id : user . id ] ) Don't allow users to edit posts they can't read. policy |&gt; allow ( :read , Post , where : [ archived : false ] ) |&gt; allow ( :edit , Post , where : [ user_id : user . id ] ) |&gt; deny ( :edit , Post , where_not : allows ( :read ) ) Example with associations Let's say we have some posts with comments. Posts are visible unless they are archived, and all comments of visible posts are also visible. To start, we can duplicate the condition: policy |&gt; allow ( :read , Post , where : [ archived : false ] ) |&gt; allow ( :read , Comment , where : [ post : [ archived : false ] ] ) If we add additional clauses to the condition for posts, however, we will have to duplicate them for comments. We can use allows instead: policy |&gt; allow ( :read , Post , where : [ archived : false ] ) |&gt; allow ( :read , Comment , where : [ post : allows ( :read ) ] ) Now let's say we add a feature that allows for draft posts, which should not be visible unless a published_at is set. We can modify only the condition for Post and that change will propogate to comments. policy |&gt; allow ( :read , Post , where : [ archived : false ] , where_not : [ published_at : nil ] ) |&gt; allow ( :read , Comment , where : [ post : allows ( :read ) ] )","ref":"Janus.Policy.html#allows/1","title":"Janus.Policy.allows/1","type":"function"},{"doc":"Registers a hook to be run prior to calling policy_for/2 . before_policy_for hooks can be used to alter the default policy or actor that is being passed into policy_for/2 . This could be used to preload required associations or fields, or to short-circuit the call entirely, immediately returning a policy without running it through policy_for/2 . before_policy_for takes a module name or a tuple containing a module name and some term. The module is expected to define a function before_policy_for/3 . The function will receive three arguments: term (defaults to :default ) policy actor and it must return one of: {:cont, policy, actor} - run any further hooks and then policy_for/2 {:halt, policy} - skip any further hooks and policy_for/2 and return policy Example defmodule Policy do use Janus before_policy_for __MODULE__ before_policy_for { __MODULE__ , :check_banned } def before_policy_for ( :default , policy , user ) do { :cont , policy , preload_required ( user ) } end def before_policy_for ( :check_banned , policy , user ) do if banned? ( user ) do { :halt , policy } else { :cont , policy , user } end end # ... end If desired, hooks can live in another module. defmodule Policy.Helpers do def before_policy_for ( :check_banned , policy , user ) do if User . banned? ( user ) do { :halt , policy } else { :cont , policy , user } end end end defmodule Policy do use Janus before_policy_for { Policy.Helpers , :check_banned } # ... end","ref":"Janus.Policy.html#before_policy_for/1","title":"Janus.Policy.before_policy_for/1","type":"macro"},{"doc":"Denies an action on the schema if matched by conditions. See the section on &quot;Using allow and deny&quot; for a description of conditions. Examples policy |&gt; allow ( :read , FirstResource ) |&gt; deny ( :read , FirstResource , where : [ scope : :private ] )","ref":"Janus.Policy.html#deny/4","title":"Janus.Policy.deny/4","type":"function"},{"doc":"Returns the policy for the given actor. This is the only callback that is required in a policy module.","ref":"Janus.Policy.html#c:policy_for/2","title":"Janus.Policy.policy_for/2","type":"callback"},{"doc":"","ref":"Janus.Policy.html#t:t/0","title":"Janus.Policy.t/0","type":"type"},{"doc":"Defines a rule for an individual schema and action.","ref":"Janus.Policy.Rule.html","title":"Janus.Policy.Rule","type":"module"},{"doc":"","ref":"Janus.Policy.Rule.html#t:t/0","title":"Janus.Policy.Rule.t/0","type":"type"},{"doc":"Creates a basic policy module. $ mix janus.gen.policy [--module MODULE] [--path PATH] Creates the following file: lib/app_name/policy.ex - A Janus policy module containing additional authorization helpers Options --module - The name of the generated module, defaults to AppName.Policy --path - The path (including filename) for the generated module, defaults to lib/app_name/policy.ex","ref":"Mix.Tasks.Janus.Gen.Policy.html","title":"Mix.Tasks.Janus.Gen.Policy","type":"task"},{"doc":"Flexible and composable authorization for resources defined by an Ecto.Schema . Janus provides an API for defining authorization policies that can be used both as filters in Ecto queries and to authorize actions on loaded resources with minimal to no duplication of authorization logic. Documentation","ref":"readme.html","title":"Janus","type":"extras"},{"doc":"Janus is pre-0.1 and can be installed by adding ex_janus as a git dependency in mix.exs : defp deps do [ { :ex_janus , github : &quot;zachallaun/ex_janus&quot; } ] end","ref":"readme.html#installation","title":"Janus - Installation","type":"extras"},{"doc":"To set up and test Janus locally: $ git clone https://github.com/zachallaun/ex_janus &amp;&amp; cd ex_janus $ mix deps.get # Set up test database and run tests with code coverage report $ mix setup $ mix t","ref":"readme.html#development","title":"Janus - Development","type":"extras"},{"doc":"All notable changes to this project will be documented in this file.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"...","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"This marks the first release of Janus. Added Janus.Policy and Janus.Authorization (behaviours) drive the definition and usage of authorization policies. [mix janus.gen.policy] Mix task for generating basic policy and helpers.","ref":"changelog.html#v0-1-0-2022-12-25","title":"Changelog - v0.1.0 (2022-12-25)","type":"extras"},{"doc":"","ref":"cheatsheet.html","title":"Cheatsheet","type":"extras"},{"doc":"Dependency mix.exs defp deps do [ { :ex_janus , github : &quot;zachallaun/ex_janus&quot; } ] end Ecto v3.9.4 or later and a database that supports lateral joins (like PostgreSQL) is required to use all Janus features. Policy module lib/my_app/policy.ex defmodule MyApp.Policy do use Janus @impl true def policy_for ( policy , user ) do policy |&gt; # authorization rules end end Your policy module is the interface used by the rest of your application. This is usually the only place you should be referring to Janus directly. Formatter (optional) .formatter.exs [ import_deps : [ :ex_janus ] ] This may be desired if you're using hooks. See Janus.Policy.before_policy_for/1 for more.","ref":"cheatsheet.html#installation-and-setup","title":"Cheatsheet - Installation and setup","type":"extras"},{"doc":"Users defmodule User do use Ecto.Schema schema &quot;users&quot; do field :email , :string field :display_name , :string field :role , Ecto.Enum , values : [ :member , :moderator , :admin ] has_many :posts , Post has_many :comments , Comment end end Posts defmodule Post do use Ecto.Schema schema &quot;posts&quot; do field :title , :string field :content , :string field :archived , :boolean , default : false belongs_to :user , User has_many :comments , Comment end end Comments defmodule Comment do use Ecto.Schema schema &quot;comments&quot; do field :content , :string belongs_to :user , User belongs_to :post , Post end end","ref":"cheatsheet.html#reference-schemas","title":"Cheatsheet - Reference schemas","type":"extras"},{"doc":"Schemas Schemas are modules that invoke use Ecto.Schema and are used to identify different resources in your application. When defining policies policy |&gt; allow ( :edit , Post , ... ) # ^^^^ When generating a query MyApp.Policy . filter_authorized ( Post , :edit , current_user ) # ^^^^ When checking whether any auth rules are defined MyApp.Policy . any_authorized? ( Post , :edit , current_user ) # ^^^^ Resources Resources are loaded structs defined by one of your schemas. When authorizing an action MyApp.Policy . authorize ( % Post { } , :edit , current_user ) # ^^^^^^^ Actors Actors are the users of your application. They can be a %User{} struct, but they don't have to be. Actors are converted to a policy using policy_for/2 , so an actor can be anything that you want to use to differentiate between types of user. They can even be a simple atom like :normal_user or :admin_user . In policy_for/2 def policy_for ( policy , % User { } ) do # ^^^^^^^ end When calling any authorization function MyApp.Policy . authorize ( % Post { } , :edit , current_user ) # ^^^^^^^^^^^^ Actions Actions are what actors do to resources in your application. Janus doesn't care how you represent actions, but atoms usually do the trick. When defining policies policy |&gt; allow ( :edit , Post , ... ) # ^^^^^ When calling any authorization function MyApp.Policy . authorize ( % Post { } , :edit , current_user ) # ^^^^^ Can be any term (except a list) policy |&gt; allow ( :edit , Post , ... ) |&gt; allow ( &quot;edit&quot; , Post , ... ) |&gt; allow ( % Action { type : :edit } , Post , ... ) # lists are special-cased to allow multiple actions to share conditions |&gt; allow ( [ :read , :edit ] , Post , ... )","ref":"cheatsheet.html#core-concepts","title":"Cheatsheet - Core concepts","type":"extras"},{"doc":"Grant permission for all resources of schema policy |&gt; allow ( :read , Post ) |&gt; allow ( :edit , Post ) |&gt; allow ( :archive , Post ) |&gt; allow ( :read , Comment ) |&gt; allow ( :edit , Comment ) Using lists of actions policy |&gt; allow ( [ :read , :edit , :archive ] , Post ) |&gt; allow ( [ :read , :edit ] , Comment ) Grant permission based on attributes policy |&gt; allow ( :read , Post , where : [ archived : false ] ) # or define using :where_not |&gt; allow ( :read , Post , where_not : [ archived : true ] ) # or override a blanket permission using deny |&gt; allow ( :read , Post ) |&gt; deny ( :read , Post , where : [ archived : true ] ) Use deny to override a previous allow policy |&gt; allow ( :read , Post ) |&gt; deny ( :read , Post , where : [ archived : true ] ) Grant permission if the user is associated with the resource def policy_for ( policy , % User { role : :member } = user ) do policy |&gt; allow ( :edit , Comment , where : [ user_id : user . id ] ) end Grant permission based on association attributes policy |&gt; allow ( :edit , Comment , where : [ user : [ role : :member ] ] ) Use allows to delegate permission to an association policy |&gt; allow ( :read , Post , where : [ archived : false ] ) |&gt; allow ( :read , Comment , where : [ post : allows ( :read ) ] ) Multiple allow combines as a logical-or # This will always allow reading all posts policy |&gt; allow ( :read , Post ) |&gt; allow ( :read , Post , where : [ archived : false ] ) # has no effect","ref":"cheatsheet.html#permitting-actions","title":"Cheatsheet - Permitting actions","type":"extras"},{"doc":"Pattern-match to give different permissions to different actors def policy_for ( policy , % User { role : :member } ) do # member permissions end def policy_for ( policy , % User { role : :moderator } ) do # moderator permissions end Delegate to context-specific policies def policy_for ( policy , actor ) do policy |&gt; CommunityForum.Policy . policy_for ( actor ) |&gt; Storefront.Policy . policy_for ( actor ) end For larger applications with well-defined boundaries, a policy can be constructed by threading it through multiple policy_for calls.","ref":"cheatsheet.html#structuring-your-policy-definition","title":"Cheatsheet - Structuring your policy definition","type":"extras"},{"doc":"Using policies in a context module Policies should most often be used in context modules, since they provide the interface to actions and resources that the rest of your application uses. defmodule MyApp.MessageBoard do @moduledoc &quot;&quot;&quot; Context module for the message board. &quot;&quot;&quot; import Ecto.Query # imports authorize, any_authorized?, filter_authorized, etc. import MyApp.Policy alias MyApp.Repo # ... end Authorizing an action on a resource def update_post ( % Post { } = post , attrs \\\\ %{ } , user_or_policy ) do case authorize ( post , :edit , user_or_policy ) do { :ok , post } -&gt; post |&gt; Post . changeset ( attrs ) |&gt; Repo . insert ( ) :error -&gt; { :error , :not_authorized } end end Fetching authorized resources def authorized_posts ( user_or_policy ) do if any_authorized? ( Post , :read , user_or_policy ) do posts = Post |&gt; filter_authorized ( :read , user_or_policy ) |&gt; Repo . all ( ) { :ok , posts } else { :error , :not_authorized } end end Use any_authorized?/3 to differentiate between a result that is empty because there are no resources that match the policy conditions and a result that is empty because the user isn't authorized to view any resources. Preloading authorized associations filter_authorized ( Post , :read , user_or_policy , preload_authorized : :comments ) filter_authorized ( Post , :read , user_or_policy , preload_authorized : [ comments : :user ] ) The :preload_authorized option can be passed to preload only those associated resources that are authorized for the given action. Applying a query to preloads latest_comment_query = from Comment , order_by : [ desc : :inserted_at ] , limit : 1 filter_authorized ( Post , :read , user_or_policy , preload_authorized : [ comments : latest_comment_query ] ) A query can be applied to associated authorized resources. It is scoped per-association, so it applies to comments of each post instead of the comments of all posts. The above would return all :read -able posts preloaded with their latest :read -able comment. You can still include nested preloads using a tuple: filter_authorized ( Post , :read , user_or_policy , preload_authorized : [ comments : { latest_comment_query , [ :user ] } ] )","ref":"cheatsheet.html#authorization","title":"Cheatsheet - Authorization","type":"extras"},{"doc":"Call policy_for/1 to get the policy for a user iex&gt; policy = MyApp.Policy . policy_for ( current_user ) % Janus.Policy { ... } Pass a policy anywhere you'd pass in an actor iex&gt; MyApp.Policy . authorize ( post , :read , policy ) { :ok , post } iex&gt; MyApp.Policy . filter_authorized ( Post , :read , policy ) % Ecto.Query { } Cache a policy in a Plug.Conn def assign_current_policy ( conn ) do %{ assigns : %{ current_user : user } } = conn conn |&gt; assign ( :current_policy , MyApp.Policy . policy_for ( user ) ) end def index ( conn , _params ) do %{ assigns : %{ current_policy : policy } } = conn # Pass the policy to your context case MessageBoard . authorized_posts ( policy ) do { :ok , posts } -&gt; ... { :error , :not_authorized } -&gt; ... end end","ref":"cheatsheet.html#caching-a-policy","title":"Cheatsheet - Caching a policy","type":"extras"}]